@@ -194,13 +194,17 @@ print.matchit <- function(x, ...) {
   nm <- is.null(x[["method"]])
   cat("A matchit object")
   cat(paste0("\n - method: ", info.to.method(info)))
-  # if (!is.null(x[["distance"]]) || info$mahalanobis || identical(info$distance, "user")) {
+
   if (!is.null(info$distance) || info$mahalanobis) {
     cat("\n - distance: ")
     if (info$mahalanobis) {
-      cat("Mahalanobis")
+      if (is.null(info$transform)) #mahvars used
+        cat("Mahalanobis")
+      else {
+        cat(capwords(gsub("_", " ", info$transform, fixed = TRUE)))
+      }
     }
-    if (!is.null(info$distance) && info$distance != "mahalanobis") {
+    if (!is.null(info$distance) && !info$distance %in% matchit_distances()) {
       if (info$mahalanobis) cat(" [matching]\n             ")
 
       if (info$distance_is_matrix) cat("User-defined (matrix)")
@@ -333,7 +337,7 @@ print.summary.matchit.subclass <- function(x, digits = max(3, getOption("digits"
 
 summary.matchit <- function(object, interactions = FALSE,
                             addlvariables = NULL, standardize = TRUE,
-                            data = NULL, pair.dist = TRUE, un = TRUE, improvement = TRUE, ...) {
+                            data = NULL, pair.dist = TRUE, un = TRUE, improvement = FALSE, ...) {
 
   #Create covariate matrix; include caliper, exact, and mahvars
 
@@ -534,7 +538,7 @@ summary.matchit <- function(object, interactions = FALSE,
 summary.matchit.subclass <- function(object, interactions = FALSE,
                                      addlvariables = NULL, standardize = TRUE,
                                      data = NULL, pair.dist = FALSE,
-                                     subclass = FALSE, un = TRUE, improvement = TRUE, ...) {
+                                     subclass = FALSE, un = TRUE, improvement = FALSE, ...) {
 
   #Create covariate matrix
   X <- get.covs.matrix(data = object$X)
