@@ -12,7 +12,7 @@ matchit2null <- function(discarded, ...) {
   return(res)
 }
 # MATCHIT method = cem--------------------------------------
-matchit2cem <- function(treat, covs, estimand = "ATT", verbose = FALSE, ...) {
+matchit2cem <- function(treat, covs, estimand = "ATT", s.weights = NULL, verbose = FALSE, ...) {
   if (length(covs) == 0) stop("Covariates must be specified in the input formula to use coarsened exact matching.", call. = FALSE)
 
   if (verbose) cat("Coarsened exact matching...\n")
@@ -23,7 +23,8 @@ matchit2cem <- function(treat, covs, estimand = "ATT", verbose = FALSE, ...) {
   estimand <- match_arg(estimand, c("ATT", "ATC", "ATE"))
 
   #Uses in-house cem, no need for cem package. See cem_matchit.R for code.
-  strat <- do.call("cem_matchit", c(list(treat = treat, X = covs, estimand = estimand),
+  strat <- do.call("cem_matchit", c(list(treat = treat, X = covs, estimand = estimand,
+                                         s.weights = s.weights),
                                     A[names(A) %in% names(formals(cem_matchit))]),
                    quote = TRUE)
 
@@ -80,7 +81,7 @@ matchit2exact <- function(treat, covs, data, estimand = "ATT", verbose = FALSE,
 
 # MATCHIT method = full-------------------------------------
 matchit2full <- function(treat, formula, data, distance, discarded,
-                         ratio = NULL, #min.controls and max.controls in attrs of replace
+                         ratio = NULL, s.weights = NULL, #min.controls and max.controls in attrs of replace
                          caliper = NULL, mahvars = NULL, exact = NULL,
                          estimand = "ATT", verbose = FALSE,
                          is.full.mahalanobis, antiexact = NULL, ...) {
@@ -91,7 +92,7 @@ matchit2full <- function(treat, formula, data, distance, discarded,
 
   A <- list(...)
 
-  fm.args <- c("omit.fraction", "mean.controls", "tol")
+  fm.args <- c("omit.fraction", "mean.controls", "tol", "solver")
   A[!names(A) %in% fm.args] <- NULL
 
   #Set max problem size to Inf and return to original value after match
@@ -112,7 +113,7 @@ matchit2full <- function(treat, formula, data, distance, discarded,
 
   treat_ <- setNames(as.integer(treat[!discarded] == focal), names(treat)[!discarded])
 
-  if (!is.null(data)) data <- data[!discarded,]
+  # if (!is.null(data)) data <- data[!discarded,]
 
   if (is.full.mahalanobis) {
     if (length(attr(terms(formula, data = data), "term.labels")) == 0) {
@@ -126,8 +127,8 @@ matchit2full <- function(treat, formula, data, distance, discarded,
 
   #Exact matching strata
   if (!is.null(exact)) {
-    ex <- factor(exactify(model.frame(exact, data = data), sep = ", ", include_vars = TRUE))
-
+    ex <- factor(exactify(model.frame(exact, data = data),
+                          sep = ", ", include_vars = TRUE)[!discarded])
     cc <- intersect(ex[treat_==1], ex[treat_==0])
     if (length(cc) == 0) stop("No matches were found.", call. = FALSE)
   }
@@ -136,30 +137,39 @@ matchit2full <- function(treat, formula, data, distance, discarded,
   }
 
   #Create distance matrix; note that Mahalanobis distance computed using entire
-  #sample, like method2nearest, as opposed to within exact strata, like optmatch.
+  #sample (minus discarded), like method2nearest, as opposed to within exact strata, like optmatch.
   if (!is.null(mahvars)) {
-    mahvars <- update(mahvars, treat_ ~ .)
-    environment(mahvars) <- sys.frame(sys.nframe())
-
-    mo <- optmatch::match_on(mahvars, data = data[names(data) != "treat_"],
-                             method = "mahalanobis")
+    transform <- if (is.full.mahalanobis) attr(is.full.mahalanobis, "transform") else "mahalanobis"
+    mahcovs <- transform_covariates(mahvars, data = data, method = transform,
+                                    s.weights = s.weights, treat = treat,
+                                    discarded = discarded)
+    mo <- eucdist_internal(mahcovs, treat)
   }
   else if (is.matrix(distance)) {
-    if (focal == 0) distance <- t(distance)
-    mo <- distance[!discarded[treat == focal], !discarded[treat != focal], drop = FALSE]
-    dimnames(mo) <- list(names(treat_)[treat_ == 1], names(treat_)[treat_ == 0])
+    mo <- distance
   }
   else {
-    mo <- optmatch::match_on(setNames(distance[!discarded], names(treat_)), z = treat_)
+    mo <- eucdist_internal(setNames(distance, names(treat)), treat)
   }
 
+  #Transpose distance mat as needed
+  if (focal == 0) mo <- t(mo)
+
+  #Remove discarded units from distance mat
+  mo <- mo[!discarded[treat == focal], !discarded[treat != focal], drop = FALSE]
+  dimnames(mo) <- list(names(treat_)[treat_ == 1], names(treat_)[treat_ == 0])
+
+  mo <- optmatch::match_on(mo, data = data[!discarded,, drop = FALSE])
+
+  #Process antiexact
   if (!is.null(antiexact)) {
     antiexactcovs <- model.frame(antiexact, data)
     for (i in seq_len(ncol(antiexactcovs))) {
-      mo <- mo + optmatch::antiExactMatch(antiexactcovs[[i]], z = treat_)
+      mo <- mo + optmatch::antiExactMatch(antiexactcovs[[i]][!discarded], z = treat_)
     }
   }
 
+  #Process caliper
   if (!is.null(caliper)) {
     if (min.controls != 0) {
       stop("Calipers cannot be used with method = \"full\" when 'min.controls' is specified.", call. = FALSE)
@@ -171,7 +181,7 @@ matchit2full <- function(treat, formula, data, distance, discarded,
     }
     for (i in seq_along(caliper)) {
       if (names(caliper)[i] != "") {
-        mo_cal <- optmatch::match_on(setNames(calcovs[,names(caliper)[i]], names(treat_)), z = treat_)
+        mo_cal <- optmatch::match_on(setNames(calcovs[!discarded, names(caliper)[i]], names(treat_)), z = treat_)
       }
       else if (is.null(mahvars) || is.matrix(distance)) {
         mo_cal <- mo
@@ -189,6 +199,8 @@ matchit2full <- function(treat, formula, data, distance, discarded,
   pair <- setNames(rep(NA_character_, length(treat)), names(treat))
   p <- setNames(vector("list", nlevels(ex)), levels(ex))
 
+  t_df <- data.frame(treat)
+
   for (e in levels(ex)) {
     if (nlevels(ex) > 1) {
       mo_ <- mo[ex[treat_==1] == e, ex[treat_==0] == e, drop = FALSE]
@@ -206,7 +218,7 @@ matchit2full <- function(treat, formula, data, distance, discarded,
                         c(list(mo_,
                                min.controls = min.controls,
                                max.controls = max.controls,
-                               data = treat), #just to get rownames; not actually used in matching
+                               data = t_df), #just to get rownames; not actually used in matching
                           A))
     },
     warning = function(w) {
@@ -244,7 +256,8 @@ matchit2full <- function(treat, formula, data, distance, discarded,
 
 # MATCHIT method = optimal----------------------------------
 matchit2optimal <- function(treat, formula, data, distance, discarded,
-                            ratio = 1, caliper = NULL, mahvars = NULL, exact = NULL,
+                            ratio = 1, s.weights = NULL, caliper = NULL,
+                            mahvars = NULL, exact = NULL,
                             estimand = "ATT", verbose = FALSE,
                             is.full.mahalanobis,  antiexact = NULL, ...) {
 
@@ -253,7 +266,7 @@ matchit2optimal <- function(treat, formula, data, distance, discarded,
   if (verbose) cat("Optimal matching... \n")
 
   A <- list(...)
-  pm.args <- c("tol")
+  pm.args <- c("tol", "solver")
   A[!names(A) %in% pm.args] <- NULL
 
   #Set max problem size to Inf and return to original value after match
@@ -274,7 +287,7 @@ matchit2optimal <- function(treat, formula, data, distance, discarded,
 
   treat_ <- setNames(as.integer(treat[!discarded] == focal), names(treat)[!discarded])
 
-  if (!is.null(data)) data <- data[!discarded,]
+  # if (!is.null(data)) data <- data[!discarded,]
 
   if (is.full.mahalanobis) {
     if (length(attr(terms(formula, data = data), "term.labels")) == 0) {
@@ -297,7 +310,8 @@ matchit2optimal <- function(treat, formula, data, distance, discarded,
 
   #Exact matching strata
   if (!is.null(exact)) {
-    ex <- factor(exactify(model.frame(exact, data = data), sep = ", ", include_vars = TRUE))
+    ex <- factor(exactify(model.frame(exact, data = data),
+                          sep = ", ", include_vars = TRUE)[!discarded])
 
     cc <- intersect(ex[treat_==1], ex[treat_==0])
     if (length(cc) == 0) stop("No matches were found.", call. = FALSE)
@@ -305,13 +319,16 @@ matchit2optimal <- function(treat, formula, data, distance, discarded,
     e_ratios <- vapply(levels(ex), function(e) sum(treat_[ex == e] == 0)/sum(treat_[ex == e] == 1), numeric(1L))
 
     if (any(e_ratios < 1)) {
-      warning(paste0("Fewer ", tc[2], " units than ", tc[1], " units in some 'exact' strata; not all ", tc[1], " units will get a match."), immediate. = TRUE, call. = FALSE)
+      warning(sprintf("Fewer %s units than %s units in some 'exact' strata; not all %s units will get a match.",
+                      tc[2], tc[1], tc[1]), immediate. = TRUE, call. = FALSE)
     }
     if (ratio > 1 && any(e_ratios < ratio)) {
       if (ratio == max.controls)
-        warning(paste0("Not all ", tc[1], " units will get ", ratio, " matches."), immediate. = TRUE, call. = FALSE)
+        warning(sprintf("Not all %s units will get %s matches.",
+                        tc[1], ratio), immediate. = TRUE, call. = FALSE)
       else
-        warning(paste0("Not enough ", tc[2], " units for an average of ", ratio, " matches per ", tc[1], " unit in all 'exact' strata."), immediate. = TRUE, call. = FALSE)
+        warning(sprintf("Not enough %s units for an average of %s matches per %s unit in all 'exact' strata.",
+                        tc[2], ratio, tc[1]), immediate. = TRUE, call. = FALSE)
     }
   }
   else {
@@ -319,38 +336,49 @@ matchit2optimal <- function(treat, formula, data, distance, discarded,
     e_ratios <- setNames(sum(treat_ == 0)/sum(treat_ == 1), levels(ex))
 
     if (e_ratios < 1) {
-      warning(paste0("Fewer ", tc[2], " units than ", tc[1], " units; not all ", tc[1], " units will get a match."), immediate. = TRUE, call. = FALSE)
+      warning(sprintf("Fewer %s units than %s units; not all %s units will get a match.",
+                      tc[2], tc[1], tc[1]), immediate. = TRUE, call. = FALSE)
     }
     else if (e_ratios < ratio) {
       if (ratio == max.controls)
-        warning(paste0("Not all ", tc[1], " units will get ", ratio, " matches."), immediate. = TRUE, call. = FALSE)
+        warning(sprintf("Not all %s units will get %s matches.",
+                        tc[1], ratio), immediate. = TRUE, call. = FALSE)
       else
-        warning(paste0("Not enough ", tc[2], " units for an average of ", ratio, " matches per ", tc[1], " unit."), immediate. = TRUE, call. = FALSE)
+        warning(sprintf("Not enough %s units for an average of %s matches per %s unit.",
+                        tc[2], ratio, tc[1]), immediate. = TRUE, call. = FALSE)
     }
   }
 
   #Create distance matrix; note that Mahalanobis distance computed using entire
-  #sample, like method2nearest, as opposed to within exact strata, like optmatch.
+  #sample (minus discarded), like method2nearest, as opposed to within exact strata, like optmatch.
   if (!is.null(mahvars)) {
-    mahvars <- update(mahvars, treat_ ~ .)
-    environment(mahvars) <- sys.frame(sys.nframe())
-
-    mo <- optmatch::match_on(mahvars, data = data[names(data) != "treat_"],
-                             method = "mahalanobis")
+    transform <- if (is.full.mahalanobis) attr(is.full.mahalanobis, "transform") else "mahalanobis"
+    mahcovs <- transform_covariates(mahvars, data = data, method = transform,
+                                    s.weights = s.weights, treat = treat,
+                                    discarded = discarded)
+    mo <- eucdist_internal(mahcovs, treat)
   }
   else if (is.matrix(distance)) {
-    if (focal == 0) distance <- t(distance)
-    mo <- distance[!discarded[treat == focal], !discarded[treat != focal], drop = FALSE]
-    dimnames(mo) <- list(names(treat_)[treat_ == 1], names(treat_)[treat_ == 0])
+    mo <- distance
   }
   else {
-    mo <- optmatch::match_on(setNames(distance[!discarded], names(treat_)), z = treat_)
+    mo <- eucdist_internal(setNames(distance, names(treat)), treat)
   }
 
+  #Transpose distance mat as needed
+  if (focal == 0) mo <- t(mo)
+
+  #Remove discarded units from distance mat
+  mo <- mo[!discarded[treat == focal], !discarded[treat != focal], drop = FALSE]
+  dimnames(mo) <- list(names(treat_)[treat_ == 1], names(treat_)[treat_ == 0])
+
+  mo <- optmatch::match_on(mo, data = data[!discarded,, drop = FALSE])
+
+  #Process antiexact
   if (!is.null(antiexact)) {
     antiexactcovs <- model.frame(antiexact, data)
     for (i in seq_len(ncol(antiexactcovs))) {
-      mo <- mo + optmatch::antiExactMatch(antiexactcovs[[i]], z = treat_)
+      mo <- mo + optmatch::antiExactMatch(antiexactcovs[[i]][!discarded], z = treat_)
     }
   }
 
@@ -358,6 +386,8 @@ matchit2optimal <- function(treat, formula, data, distance, discarded,
   pair <- setNames(rep(NA_character_, length(treat)), names(treat))
   p <- setNames(vector("list", nlevels(ex)), levels(ex))
 
+  t_df <- data.frame(treat)
+
   for (e in levels(ex)) {
     if (nlevels(ex) > 1) {
       mo_ <- mo[ex[treat_==1] == e, ex[treat_==0] == e, drop = FALSE]
@@ -395,7 +425,7 @@ matchit2optimal <- function(treat, formula, data, distance, discarded,
                                mean.controls = ratio_,
                                min.controls = min.controls_,
                                max.controls = max.controls_,
-                               data = treat), #just to get rownames; not actually used in matching
+                               data = t_df), #just to get rownames; not actually used in matching
                           A))
     },
     warning = function(w) {
@@ -458,9 +488,12 @@ matchit2genetic <- function(treat, data, distance, discarded,
 
   if (!replace) {
     if (sum(!discarded & treat != focal) < sum(!discarded & treat == focal)) {
-      warning("Fewer ", tc[2], " units than ", tc[1], " units; not all ", tc[1], " units will get a match.", immediate. = TRUE, call. = FALSE)    }
+      warning(sprintf("Fewer %s units than %s units; not all %s units will get a match.",
+                      tc[2], tc[1], tc[1]), immediate. = TRUE, call. = FALSE)
+    }
     else if (sum(!discarded & treat != focal) < sum(!discarded & treat == focal)*ratio) {
-      stop(paste0("Not enough ", tc[2], " units for ", ratio, " matches for each ", tc[1], " unit."), call. = FALSE)
+      stop(sprintf("Not enough %s units for %s matches for each %s unit.",
+                   tc[2], ratio, tc[1]), call. = FALSE)
     }
   }
 
@@ -471,27 +504,24 @@ matchit2genetic <- function(treat, data, distance, discarded,
 
   if (is.null(names(treat))) names(treat) <- seq_len(n.obs)
 
-  if (!is.null(distance)) {
-    if (is.null(m.order)) m.order <- if (estimand == "ATC") "smallest" else "largest"
-    else m.order <- match_arg(m.order, c("largest", "smallest", "random", "data"))
-    ord <- switch(m.order,
-                  "largest" = order(distance, decreasing = TRUE),
-                  "smallest" = order(distance),
-                  "random" = sample(seq_len(n.obs), n.obs, replace = FALSE),
-                  "data" = seq_len(n.obs))
-  }
-  else {
-    m.order <- match_arg(m.order, c("data", "random"))
-    ord <- switch(m.order,
-                  "random" = sample(seq_len(n.obs), n.obs, replace = FALSE),
-                  "data" = seq_len(n.obs))
+  m.order <- {
+    if (is.null(distance)) match_arg(m.order, c("data", "random"))
+    else if (!is.null(m.order)) match_arg(m.order, c("largest", "smallest", "random", "data"))
+    else if (estimand == "ATC") "smallest"
+    else "largest"
   }
+
+  ord <- switch(m.order,
+                "largest" = order(distance, decreasing = TRUE),
+                "smallest" = order(distance),
+                "random" = sample.int(n.obs),
+                "data" = seq_len(n.obs))
   ord <- ord[!ord %in% which(discarded)]
 
   #Create X (matching variables) and covs_to_balance
   covs_to_balance <- get.covs.matrix(formula, data = data)
   if (!is.null(mahvars)) {
-    X <- get.covs.matrix(mahvars, data = data)
+    X <- get.covs.matrix.for.dist(mahvars, data = data)
   }
   else if (is.full.mahalanobis) {
     X <- covs_to_balance
@@ -574,7 +604,7 @@ matchit2genetic <- function(treat, data, distance, discarded,
 
   treat_ <- treat[ord]
   covs_to_balance <- covs_to_balance[ord,,drop = FALSE]
-  X_ <- X[ord,,drop = FALSE]
+  X <- X[ord,,drop = FALSE]
   if (!is.null(s.weights)) s.weights <- s.weights[ord]
 
   if (!is.null(antiexact)) {
@@ -595,7 +625,7 @@ matchit2genetic <- function(treat, data, distance, discarded,
   if (use.genetic) {
     withCallingHandlers({
       g.out <- do.call(Matching::GenMatch,
-                       c(list(Tr = treat_, X = X_, BalanceMatrix = covs_to_balance,
+                       c(list(Tr = treat_, X = X, BalanceMatrix = covs_to_balance,
                               M = ratio, exact = exact.log, caliper = cal,
                               replace = replace, estimand = "ATT", ties = FALSE,
                               CommonSupport = FALSE, verbose = verbose,
@@ -626,13 +656,13 @@ matchit2genetic <- function(treat, data, distance, discarded,
 
   # if (!isFALSE(A$use.Match)) {
   withCallingHandlers({
-    m.out <- Matching::Match(Tr = treat_, X = X_,
+    m.out <- Matching::Match(Tr = treat_, X = X,
                              M = ratio, exact = exact.log, caliper = cal,
                              replace = replace, estimand = "ATT", ties = FALSE,
                              weights = s.weights, CommonSupport = FALSE, Weight = 3,
                              Weight.matrix = if (use.genetic) g.out
-                             else if (is.null(s.weights)) generalized_inverse(cor(X_))
-                             else generalized_inverse(cov.wt(X_, s.weights, cor = TRUE)$cor),
+                             else if (is.null(s.weights)) generalized_inverse(cor(X))
+                             else generalized_inverse(cov.wt(X, s.weights, cor = TRUE)$cor),
                              restrict = A[["restrict"]], version = "fast")
   },
   warning = function(w) {
@@ -706,7 +736,7 @@ matchit2nearest <-  function(treat, data, distance, discarded,
                              caliper = NULL, mahvars = NULL, exact = NULL,
                              formula = NULL, estimand = "ATT", verbose = FALSE,
                              is.full.mahalanobis, fast = TRUE,
-                             antiexact = NULL, ...){
+                             antiexact = NULL, unit.id = NULL, ...){
 
   if (verbose) {
     if (fast) check.package("RcppProgress")
@@ -726,8 +756,16 @@ matchit2nearest <-  function(treat, data, distance, discarded,
 
   treat <- setNames(as.integer(treat == focal), names(treat))
 
+  if (is.full.mahalanobis) {
+    if (length(attr(terms(formula, data = data), "term.labels")) == 0) {
+      stop("Covariates must be specified in the input formula when distance = \"mahalanobis\".", call. = FALSE)
+    }
+    mahvars <- formula
+  }
+
   n.obs <- length(treat)
   n1 <- sum(treat == 1)
+  n0 <- n.obs - n1
 
   lab <- names(treat)
   lab1 <- lab[treat == 1]
@@ -736,32 +774,22 @@ matchit2nearest <-  function(treat, data, distance, discarded,
     names(distance) <- names(treat)
   }
 
+  min.controls <- attr(ratio, "min.controls")
+  max.controls <- attr(ratio, "max.controls")
+
   mahcovs <- mahSigma_inv <- distance_mat <- NULL
-  if (is.full.mahalanobis) {
-    mahcovs <- get.covs.matrix(formula, data)
-    if (ncol(mahcovs) == 0) stop("Covariates must be specified in the input formula when distance = \"mahalanobis\".", call. = FALSE)
-    if (is.null(s.weights))
-      mahSigma_inv <- generalized_inverse(cov(mahcovs))
-    else
-      mahSigma_inv <- generalized_inverse(cov.wt(mahcovs, s.weights)$cov)
-    mahcovs <- tcrossprod(mahcovs, chol2(mahSigma_inv))
-  }
-  else if (!is.null(mahvars)) {
-    mahcovs <- get.covs.matrix(mahvars, data)
-    if (is.null(s.weights))
-      mahSigma_inv <- generalized_inverse(cov(mahcovs))
-    else
-      mahSigma_inv <- generalized_inverse(cov.wt(mahcovs, s.weights)$cov)
-    mahcovs <- tcrossprod(mahcovs, chol2(mahSigma_inv))
+  if (!is.null(mahvars)) {
+    transform <- if (is.full.mahalanobis) attr(is.full.mahalanobis, "transform") else "mahalanobis"
+    mahcovs <- transform_covariates(mahvars, data = data, method = transform,
+                                    s.weights = s.weights, treat = treat,
+                                    discarded = discarded)
   }
   else if (is.matrix(distance)) {
     distance_mat <- distance
     distance <- NULL
   }
 
-  min.controls <- attr(ratio, "min.controls")
-  max.controls <- attr(ratio, "max.controls")
-
+  #Process caliper
   if (!is.null(caliper)) {
     if (any(names(caliper) != "")) {
       caliper.covs <- caliper[names(caliper) != ""]
@@ -783,6 +811,7 @@ matchit2nearest <-  function(treat, data, distance, discarded,
     caliper.covs.mat <- NULL
   }
 
+  #Process antiexact
   if (!is.null(antiexact)) {
     antiexactcovs <- model.frame(antiexact, data)
     antiexactcovs <- do.call("cbind", lapply(seq_len(ncol(antiexactcovs)), function(i) {
@@ -799,6 +828,19 @@ matchit2nearest <-  function(treat, data, distance, discarded,
     m.order <- "data"
   }
 
+  if (!is.null(unit.id) && reuse.max < n1) {
+    unit.id <- process.variable.input(unit.id, data)
+    unit.id <- factor(exactify(model.frame(unit.id, data = data),
+                             nam = lab, sep = ", ", include_vars = TRUE))
+    num_ctrl_unit.ids <- length(unique(unit.id[treat == 0]))
+
+    #If each control unit is a unit.id, unit.ids are meaningless
+    if (num_ctrl_unit.ids == n0) unit.id <- NULL
+  }
+  else {
+    unit.id <- NULL
+  }
+
   if (!is.null(exact)) {
     ex <- factor(exactify(model.frame(exact, data = data), nam = lab, sep = ", ", include_vars = TRUE))
 
@@ -807,35 +849,49 @@ matchit2nearest <-  function(treat, data, distance, discarded,
 
     if (reuse.max < n1) {
 
-      e_ratios <- vapply(levels(ex), function(e) sum(treat[ex == e] == 0)*(reuse.max/sum(treat[ex == e] == 1)), numeric(1L))
+      e_ratios <- vapply(levels(ex), function(e) {
+        if (is.null(unit.id)) sum(treat[ex == e] == 0)*(reuse.max/sum(treat[ex == e] == 1))
+        else length(unique(unit.id[treat == 0 & ex == e]))*(reuse.max/sum(treat[ex == e] == 1))
+      }, numeric(1L))
 
       if (any(e_ratios < 1)) {
-        warning(paste0("Fewer ", tc[2], " units than ", tc[1], " units in some 'exact' strata; not all ", tc[1], " units will get a match."), immediate. = TRUE, call. = FALSE)
+        warning(sprintf("Fewer %s units than %s units in some 'exact' strata; not all %s units will get a match.",
+                       tc[2], tc[1], tc[1]), immediate. = TRUE, call. = FALSE)
       }
       if (ratio > 1 && any(e_ratios < ratio)) {
         if (is.null(max.controls) || ratio == max.controls)
-          warning(paste0("Not all ", tc[1], " units will get ", ratio, " matches."), immediate. = TRUE, call. = FALSE)
+          warning(sprintf("Not all %s units will get %s matches.",
+                          tc[1], ratio), immediate. = TRUE, call. = FALSE)
         else
-          warning(paste0("Not enough ", tc[2], " units for an average of ", ratio, " matches per ", tc[1], " unit in all 'exact' strata."), immediate. = TRUE, call. = FALSE)
+          warning(sprintf("Not enough %s units for an average of %s matches per %s unit in all 'exact' strata.",
+                          tc[2], ratio, tc[1]), immediate. = TRUE, call. = FALSE)
       }
     }
   }
   else {
     ex <- NULL
-    # ex <- factor(rep("_", length(treat)), levels = "_")
 
     if (reuse.max < n1) {
 
-      e_ratios <- setNames(sum(treat == 0)*(as.numeric(reuse.max)/sum(treat == 1)), levels(ex))
+      e_ratios <- {
+        if (is.null(unit.id)) as.numeric(reuse.max)*n0/n1
+        else as.numeric(reuse.max)*num_ctrl_unit.ids/n1
+      }
 
       if (e_ratios < 1) {
-        warning(paste0("Fewer ", tc[2], " units than ", tc[1], " units; not all ", tc[1], " units will get a match."), immediate. = TRUE, call. = FALSE)
+        warning(sprintf("Fewer %s %s than %s units; not all %s units will get a match.",
+                        tc[2], if (is.null(unit.id)) "units" else "unit IDs", tc[1], tc[1]),
+                immediate. = TRUE, call. = FALSE)
       }
       else if (e_ratios < ratio) {
         if (is.null(max.controls) || ratio == max.controls)
-          warning(paste0("Not all ", tc[1], " units will get ", ratio, " matches."), immediate. = TRUE, call. = FALSE)
+          warning(sprintf("Not all %s units will get %s matches.",
+                          tc[1], ratio),
+                  immediate. = TRUE, call. = FALSE)
         else
-          warning(paste0("Not enough ", tc[2], " units for an average of ", ratio, " matches per ", tc[1], " unit."), immediate. = TRUE, call. = FALSE)
+          warning(sprintf("Not enough %s %s for an average of %s matches per %s unit.",
+                          tc[2], if (is.null(unit.id)) "units" else "unit IDs", ratio, tc[1]),
+                  immediate. = TRUE, call. = FALSE)
       }
     }
   }
@@ -844,7 +900,8 @@ matchit2nearest <-  function(treat, data, distance, discarded,
   #Each treated unit get its own value of ratio
   if (!is.null(max.controls)) {
     if (is.null(distance)) {
-      if (is.null(distance_mat)) stop("'distance' cannot be \"mahalanobis\" for variable ratio matching.", call. = FALSE)
+      if (is.full.mahalanobis) stop(sprintf("'distance' cannot be \"%s\" for variable ratio matching.",
+                                            transform), call. = FALSE)
       else stop("'distance' cannot be supplied as a matrix for variable ratio matching.", call. = FALSE)
     }
 
@@ -876,30 +933,27 @@ matchit2nearest <-  function(treat, data, distance, discarded,
   }
   max_rat <- max(ratio)
 
-  if (!is.null(distance)) {
-    if (is.null(m.order)) m.order <- if (estimand == "ATC") "smallest" else "largest"
-    else m.order <- match_arg(m.order, c("largest", "smallest", "random", "data"))
-  }
-  else {
-    #Mahalanobis or distance matrix
-    m.order <- match_arg(m.order, c("data", "random"))
+  m.order <- {
+    if (is.null(distance)) match_arg(m.order, c("data", "random"))
+    else if (!is.null(m.order)) match_arg(m.order, c("largest", "smallest", "random", "data"))
+    else if (estimand == "ATC") "smallest"
+    else "largest"
   }
 
-  if (is.null(ex)) {
+  if (is.null(ex) || !is.null(unit.id)) {
     ord <- switch(m.order,
                   "largest" = order(distance[treat == 1], decreasing = TRUE),
                   "smallest" = order(distance[treat == 1], decreasing = FALSE),
-                  "random" = sample(seq_len(n1), n1, replace = FALSE),
+                  "random" = sample.int(n1),
                   "data" = seq_len(n1))
-    mm <- matrix(NA_integer_, nrow = n1, ncol = max_rat, dimnames = list(lab1))
-
-    mm <- nn_matchC(mm, treat, ord, ratio, max_rat, discarded, reuse.max, distance, distance_mat, ex, caliper.dist,
-                    caliper.covs, caliper.covs.mat, mahcovs, antiexactcovs, verbose)
+    mm <- nn_matchC(treat, ord, ratio, max_rat, discarded, reuse.max, distance, distance_mat, ex, caliper.dist,
+                    caliper.covs, caliper.covs.mat, mahcovs, antiexactcovs, unit.id, verbose)
   }
   else {
     mm_list <- lapply(levels(ex), function(e) {
       if (verbose) {
-        cat(paste0("Matching subgroup ", match(e, levels(ex)), "/", nlevels(ex), ": ", e, "...\n"))
+        cat(sprintf("Matching subgroup %s/%s: %s...\n",
+                    match(e, levels(ex)), nlevels(ex), e))
       }
 
       distance_ <- caliper.covs.mat_ <- mahcovs_ <- distance_mat_ <- NULL
@@ -920,14 +974,13 @@ matchit2nearest <-  function(treat, data, distance, discarded,
       ord_ <- switch(m.order,
                      "largest" = order(distance_[treat_ == 1], decreasing = TRUE),
                      "smallest" = order(distance_[treat_ == 1], decreasing = FALSE),
-                     "random" = sample(seq_len(n1_), n1_, replace = FALSE),
+                     "random" = sample.int(n1_),
                      "data" = seq_len(n1_))
 
-      mm_ <- matrix(NA_integer_, nrow = sum(treat_==1), ncol = max_rat, dimnames = list(names(treat_)[treat_ == 1]))
-
-      mm_ <- nn_matchC(mm_, treat_, ord_, ratio_, max_rat, discarded_, reuse.max, distance_, distance_mat_, NULL, caliper.dist,
-                       caliper.covs, caliper.covs.mat_, mahcovs_, antiexactcovs, verbose)
+      mm_ <- nn_matchC(treat_, ord_, ratio_, max_rat, discarded_, reuse.max, distance_, distance_mat_, NULL, caliper.dist,
+                       caliper.covs, caliper.covs.mat_, mahcovs_, antiexactcovs, NULL, verbose)
 
+      #Ensure matched indices correspond to indices in full sample, not subgroup
       mm_[] <- seq_along(treat)[.e][mm_]
       mm_
     })
@@ -992,7 +1045,8 @@ matchit2subclass <- function(treat, distance, discarded,
     else {
       sub.by <- sub.by.choices[pmatch(sub.by, sub.by.choices)]
       estimand <- switch(sub.by, "treat" = "ATT", "control" = "ATC", "ATE")
-      warning(paste0("'sub.by' is deprecated and has been replaced with 'estimand'. Setting 'estimand' to \"", estimand, "\"."), call. = FALSE, immediate. = TRUE)
+      warning(sprintf("'sub.by' is deprecated and has been replaced with 'estimand'. Setting 'estimand' to \"%s\".",
+                      estimand), call. = FALSE, immediate. = TRUE)
     }
   }
   else {
@@ -1045,7 +1099,6 @@ matchit2subclass <- function(treat, distance, discarded,
   if (verbose) cat("Calculating matching weights... ")
 
   res <- list(subclass = psclass, q.cut = q,
-              qn = qn(treat, psclass, discarded),
               weights = weights.subclass(psclass, treat, estimand))
 
   if (verbose) cat("Done.\n")
