@@ -2,7 +2,7 @@
 knitr::opts_chunk$set(echo = TRUE, eval=T)
 options(width = 200, digits= 4)
 
-#Generatng data similar to Austin (2009) for demonstrating treatment effect estimation
+#Generating data similar to Austin (2009) for demonstrating treatment effect estimation
 gen_X <- function(n) {
   X <- matrix(rnorm(9 * n), nrow = n, ncol = 9)
   X[,5] <- as.numeric(X[,5] < .5)
@@ -519,13 +519,13 @@ coxph(Surv(Y_S) ~ A, data = md, robust = TRUE,
 ## -----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 mF <- matchit(A ~ X1 + X2 + X3 + X4 + X5 + 
                  X6 + X7 + X8 + X9, data = d,
-              method = "full", estimand = "ATT")
+              method = "nearest")
 
 md <- match.data(mF)
 
-fitcond <- lm(Y_B ~ A + X1 + X2 + X3 + X4 + X5 + 
+fitcond <- glm(Y_B ~ A + X1 + X2 + X3 + X4 + X5 + 
                  X6 + X7 + X8 + X9, data = md, 
-             weights = weights)
+             weights = weights, family = binomial)
 
 coeftest(fitcond, vcov. = vcovCL, cluster = ~subclass)["A",,drop = FALSE]
 exp(coef(fitcond)["A"])
@@ -567,7 +567,7 @@ fitS <- lm(Y_C ~ A * X5, data = mdS, weights = weights)
 coeftest(fitS, vcov. = vcovCL, cluster = ~subclass)
 
 ## ---- eval = FALSE------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-#  #Generatng data similar to Austin (2009) for demonstrating treatment effect estimation
+#  #Generating data similar to Austin (2009) for demonstrating treatment effect estimation
 #  gen_X <- function(n) {
 #    X <- matrix(rnorm(9 * n), nrow = n, ncol = 9)
 #    X[,5] <- as.numeric(X[,5] < .5)
